# frozen_string_literal: true
require 'eaternet/socrata'

module Eaternet
  module Agencies

    # @see https://data.austintexas.gov/dataset/Restaurant-Inspection-Scores/ecmv-9xxi
    class Austin < Eaternet::Lives_1_0::Adapter
      include Eaternet::Loggable
      include Eaternet::Lives_1_0::CsvParser


      # Create an Austin data-source, ready for querying.
      #
      # @example
      #   austin = Eaternet::Austin.new
      #
      # @param [String] csv_path for unit testing
      def initialize(csv_path: nil)
        @table_file = csv_path
      end


      # @example Print the number of restaurants in the city.
      #   puts austin.businesses.count
      #
      # @return [Enumerable<Business>]
      def businesses
        map_csv { |row| try_to_create(:business, from_csv_row: row) }
          .uniq
          .compact
      end


      def inspections
        map_csv { |row| try_to_create(:inspection, from_csv_row: row) }
          .compact
      end


      def violations
        fail Eaternet::UnsupportedError, "Austin doesn't publish violation data"
      end


      def legends
        fail Eaternet::UnsupportedError, 'Austin does not assign letter grades'
      end


      def feed_info
        Eaternet::Lives_1_0::FeedInfo.new do |fi|
          fi.feed_date = Date.today
          fi.feed_version = '1.0'
          fi.municipality_name = 'Austin'
          fi.municipality_url = 'http://www.austintexas.gov/department/health'
        end
      end



      def inspection(row)
        Eaternet::Lives_1_0::Inspection.new do |i|
          i.business_id = row['Facility ID']
          i.date =        Date.strptime(row['Inspection Date'], '%m/%d/%Y')
          i.score =       row['Score'].to_i
        end
      end


      def business(row)
        Eaternet::Lives_1_0::Business.new do |b|
          fail ArgumentError, 'No address found' if row['Address'].nil?

          # Address is a multi-line, multi-value cell
          address_lines = row['Address'].split("\n")
          if address_lines.size != 3
            fail ArgumentError,
                 "Address doesn't have three lines: {row['Address']}"
          end

          address = address_lines.first
          city    = address_lines[1].split(',').first
          address_lines[2] =~ /^\(([^,]+), (.+)\)$/
          lat = Regexp.last_match(1)
          lon = Regexp.last_match(2)

          b.business_id = row['Facility ID']
          b.name =        row['Restaurant Name']
          b.address =     address
          b.city =        city
          b.postal_code = row['Zip Code']
          b.state       = 'TX'
          b.latitude    = lat
          b.longitude   = lon
        end
      end



      def table_file
        Austin.download_via_url
      end

      def self.download_via_url
        Eaternet::Util.download_and_cache(source: csv_url, dest: Tempfile.new('austin'))
      end

      def self.csv_url
        Eaternet::Socrata.csv_url domain: 'data.austintexas.gov', dataset: 'ecmv-9xxi'
      end

      def adapter_name
        'Austin'
      end
    end
  end
end
