# frozen_string_literal: true
require 'csv'
require 'json'
require 'logger'
require 'set'
require 'tempfile'

require 'ext/enumerator'

require 'eaternet/lives_1_0'
require 'eaternet/lives_1_0/csv_parser'
require 'eaternet/loggable'


module Eaternet
  module Agencies
    # A data source for New York City food service health inspections.  It
    # retrieves the latest CSV export from [the official
    # source](https://data.cityofnewyork.us/Health/DOHMH-New-York-City-Restaurant-Inspection-Results/xx67-kt59)
    # and makes it easy to work with. See our [NYC wiki page](https://github.com/eaternet/adapters/wiki/Agency:-New-York-City)
    # for details about the data set.
    #
    # Output is produced in the [LIVES 1.0](http://www.yelp.com/healthscores)
    # format developed by Yelp and the cities of San Francisco and New York.
    #
    # The downloaded CSV is cached for twelve hours in a temporary file.
    #
    # @example Print the names of all the restaurants in New York City
    #   require 'eaternet'
    #   nyc = Eaternet::Nyc.new
    #   puts nyc.businesses.map(&:name)
    #
    # @example Compute the average inspection score
    #   # The library is optimized for memory use at the expense
    #   # of speed. E.g., each call to #inspections will iterate
    #   # through the raw CSV. So here, we first retrieve the
    #   # inspections into an array.
    #   inspections = nyc.inspections.to_a
    #   sum = inspections
    #           .map(&:score)
    #           .reduce(0, :+)
    #   count = inspections.count
    #
    #   puts "Average inspection score: #{sum / count}"
    #
    # @example See the Instance Method Details for more examples.
    #
    # @see https://data.cityofnewyork.us/Health/DOHMH-New-York-City-Restaurant-Inspection-Results/xx67-kt59 NYC health scores official data, hosted on Socrata
    class Nyc < Eaternet::Lives_1_0::Adapter
      include Eaternet::Loggable
      include Eaternet::Lives_1_0::CsvParser

      # Create an NYC data-source, ready for querying.
      #
      # @example
      #   nyc = Eaternet::Nyc.new
      #
      # @param [String] csv_path for unit testing
      def initialize(csv_path: nil)
        @fixture_table_file = csv_path
      end

      # @example Print the number of restaurants in New York.
      #   puts nyc.businesses.count
      #
      # @return [Enumerable<Business>]
      def businesses
        map_csv { |row| try_to_create(:business, from_csv_row: row) }
          .uniq
          .compact
      end

      # @example Compute the average inspection score for NYC.
      #   # The library is optimized for memory use at the expense
      #   # of speed. E.g., each call to #inspections will iterate
      #   # through the raw CSV. So here, we first retrieve the
      #   # inspections into an array.
      #   inspections = nyc.inspections.to_a
      #   sum = inspections
      #           .map(&:score)
      #           .reduce(0, :+)
      #   count = inspections.count
      #
      #   puts "Average inspection score: #{sum / count}"
      #
      # @return [Enumerable<Inspection>]
      def inspections
        map_csv { |row| skip_inspection?(row) ? nil : try_to_create(:inspection, from_csv_row: row) }
          .uniq
          .compact
      end

      # @return [Enumerable<Violation>]
      def violations
        map_csv { |row| skip_violation?(row) ? nil : violation(row) }
          .compact
      end

      # @example Print the name & URL of NYC's health agency.
      #   puts nyc.feed_info.municipality_name
      #   puts nyc.feed_info.municipality_url
      #
      # @return [FeedInfo]
      def feed_info
        # Anyone know a contact email?
        Eaternet::Lives_1_0::FeedInfo.new do |fi|
          fi.feed_date = Date.today
          fi.feed_version = '1.0'
          fi.municipality_name = 'New York City'
          fi.municipality_url = 'http://www.nyc.gov/html/doh/html/services/restaurant-inspection.shtml'
        end
      end

      # @return [Enumerable<Legend>]
      def legends
        Eaternet::Lives_1_0::LegendGroup.new do |lg|
          lg.legends = [
            Eaternet::Lives_1_0::Legend.new do |l|
              l.minimum_score = 87
              l.maximum_score = 100
              l.description =   'A'
            end,
            Eaternet::Lives_1_0::Legend.new do |l|
              l.minimum_score = 73
              l.maximum_score = 86
              l.description =   'B'
            end,
            Eaternet::Lives_1_0::Legend.new do |l|
              l.minimum_score = 0
              l.maximum_score = 72
              l.description =   'C'
            end
          ]
        end.legends
      end


      # @private
      def business(row)
        address = "#{row['BUILDING']} #{row['STREET']}"

        Eaternet::Lives_1_0::Business.new do |b|
          b.business_id =  business_id(row)
          b.name =         row['DBA']
          b.address =      address
          b.city =         row['BORO']
          b.postal_code =  row['ZIPCODE']
          b.state =        'NY'
          b.phone_number = row['PHONE']
        end
      end

      private

      def violation(row)
        Eaternet::Lives_1_0::Violation.new do |v|
          v.business_id = business_id(row)
          v.date =        Date.strptime(row['INSPECTION DATE'], '%m/%d/%Y')
          v.code =        violation_code(row)
          v.description = row['VIOLATION DESCRIPTION']
        end
      end

      def inspection(row)
        Eaternet::Lives_1_0::Inspection.new do |i|
          i.business_id = business_id(row)
          i.date =        Date.strptime(row['INSPECTION DATE'], '%m/%d/%Y')
          i.score =       100 - row['SCORE'].to_i
        end
      end

      def skip_violation?(row)
        transfat_inspection?(row) || violation_code(row).nil?
      end

      def skip_inspection?(row)
        transfat_inspection?(row)
      end

      def business_id(row)
        row['CAMIS']
      end

      def violation_code(row)
        row['VIOLATION CODE']
      end

      def transfat_inspection?(row)
        if row['INSPECTION TYPE']
          row['INSPECTION TYPE'].include?('Trans Fat')
        else
          false
        end
      end

      def table_file
        @fixture_table_file || Nyc.download_via_url
      end

      # Fastest method for downloading all data but may be non-standard
      # for Socrata.
      def self.download_via_url
        Eaternet::Util.download_and_cache(source: csv_url, dest: Tempfile.new('nyc'))
      end

      def self.csv_url
        Eaternet::Socrata.csv_url domain: 'data.cityofnewyork.us', dataset: 'xx67-kt59'
      end

      def adapter_name
        'NYC'
      end
    end
  end
end
