require 'eaternet/lives_1_0/adapter'
require 'eaternet/validated_object'

module Eaternet
  module Lives_1_0
    # A food service establishment, e.g. a restaurant.
    #
    # @!attribute [rw] business_id
    #   Unique identifier for the business. For many cities,
    #   this may be the license number. Required.
    #   @return [String]
    #
    # @!attribute [rw] name
    #   Common name of the business. Required.
    #   @return [String]
    #
    # @!attribute [rw] address
    #   Street address of the business. For example: 706 Mission St.
    #   Required.
    #   @return [String]
    #
    # @!attribute [rw] city
    #   City of the business. This field must be included if the
    #   file contains businesses from multiple cities.
    #   @return [String]
    #
    # @!attribute [rw] state
    #   State or province for the business. In the U.S. this should
    #   be the two-letter code for the state. Optional.
    #   @return [String]
    #
    # @!attribute [rw] postal_code
    #   Zip code or other postal code. Optional.
    #   @return [String]
    #
    # @!attribute [rw] latitude
    #   Latitude of the business. This field must be a valid WGS 84
    #   latitude. For example: 37.7859547. Optional.
    #   @return [Float]
    #
    # @!attribute [rw] longitude
    #   Longitude of the business. This field must be a valid WGS 84
    #   longitude. For example: -122.4024658. Optional.
    #   @return [Float]
    #
    # @!attribute [rw] phone_number
    #   Phone number for a business including country specific dialing
    #   information. For example: +14159083801
    #   @return [String]
    #
    # @see http://www.yelp.com/healthscores#businesses LIVES / Business
    #   specification
    class Business < ValidatedObject
      attr_accessor :business_id, :name, :address, :city, :state,
                    :postal_code, :latitude, :longitude, :phone_number

      validates :business_id, :name, :address,
                type: String,
                presence: true

      validates :city, :state, :postal_code, :phone_number,
                type: String,
                allow_nil: true

      validates :latitude,
                numericality:
                  {
                    greater_than_or_equal_to: -90,
                    less_than_or_equal_to: 90
                  },
                allow_nil: true

      validates :longitude,
                numericality:
                  {
                    greater_than_or_equal_to: -180,
                    less_than_or_equal_to: 180
                  },
                allow_nil: true

      def latitude
        Float(@latitude) unless @latitude.nil?
      end

      def longitude
        Float(@longitude) unless @latitude.nil?
      end

      def ==(other)
        business_id == other.business_id
      end

      def eql?(other)
        self == other
      end

      def hash
        business_id.hash
      end

      # @return [String]
      def to_s
        "Business #{business_id}"
      end
    end
  end
end
