require 'eaternet/validated_object'

module Eaternet
  module Lives_1_0
    # Information about an inspectors’ visit to a businesses.
    #
    # @see http://www.yelp.com/healthscores#inspections LIVES / Inspections
    #   specification
    class Inspection < ValidatedObject
      ZERO_TO_ONE_HUNDRED_AND_BLANK = (0..100).to_a + ['']

      # Unique identifier of the business for which this inspection
      # was done.
      #
      # @required Yes
      # @return [String]
      attr_accessor :business_id
      validates     :business_id, type: String, presence: true

      # Inspection score on a 0-100 scale. 100 is the highest score. This
      # column must always be present in inspections.csv. However, it can be
      # safely left blank for inspection rows that don’t have an associated
      # score. (For example, some municipalities don’t associate a follow-up
      # inspection with a score.)
      #
      # @required No
      # @return [Integer] if it's a scored inspection
      # @return [String] if it's an un-scored inspection, then the return
      #   value will be an empty string.
      attr_accessor :score
      validates     :score,
                    inclusion: { in: ZERO_TO_ONE_HUNDRED_AND_BLANK },
                    allow_nil: true
      def score
        # noinspection RubyResolve
        @score.nil? ? '' : @score
      end

      # Date of the inspection.
      #
      # @required Yes
      # @return [Date]
      attr_accessor :date
      validates     :date, type: Date, presence: true

      # Single line description containing details on the outcome of an
      # inspection. Use of this field is only encouraged if no violations
      # are provided.
      #
      # @required No
      # @return [String]
      attr_accessor :description

      # String representing the type of inspection. Must be (initial,
      # routine, followup, complaint).
      #
      # @required No
      # @return [String]
      attr_accessor :type
      validates     :type,
                    format: { with: /initial|routine|followup|complaint/i },
                    allow_nil: true

      # @return [String]
      def to_s
        "Inspection #{business_id}/#{date}/#{score}"
      end

      def ==(other)
        business_id == other.business_id && date == other.date
      end

      def eql?(other)
        self == other
      end

      def hash
        "#{business_id} #{date}".hash
      end
    end
  end
end
